//
//  KRTableView.m
//  CustomTableView
//
//  Created by Ken Runyon on 8/9/13.
//  Copyright (c) 2013 Ken Runyon. All rights reserved.
//

#import "KRTableView.h"
#import "KRTableViewCell.h"

@implementation KRTableView

@synthesize tableDelegate;
@synthesize tableDataSource;
@synthesize rowHeight;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        [self setBackgroundColor:[UIColor blueColor]];
        
        // no zooming
        [self setMinimumZoomScale:1.0f];
        [self setMaximumZoomScale:1.0f];
        
        // allow scrolling to any point in the table
        [self setPagingEnabled:NO];
        
        // array of visible cells
        visibleCells = [[NSMutableArray alloc] init];
        recycledCells = [[NSMutableArray alloc] init];
        
        // Be delegate for scroll notifications
        [self setDelegate:self];
        
    }
    return self;
}

- (void)dealloc
{
    tableDelegate = nil;
    tableDataSource = nil;
    
    recycledCells = nil;
    visibleCells = nil;
}

//- (void)layoutSubviews
- (void)reloadData
{
    // Get the bounds of this table view
    CGRect viewBounds = [self bounds];
    
    // Assuming fixed row height! Get height of first row
    rowHeight = [tableDelegate tableView:self heightForRowAtIndexPath:[NSIndexPath indexPathWithIndex:0]];
    
    // Get total number of items in the data source
    NSInteger count = [tableDataSource tableView:self numberOfRowsInSection:1];
    
    // Set the virtual scroll area for the table
    CGFloat virtualHeight = rowHeight * count;
    [self setContentSize:CGSizeMake(viewBounds.size.width, virtualHeight)];
    
    
    // Recycle any cells that have gone out off the view visible area
    for ( id cell in visibleCells )
    {
        // For each cell that is no longer visible, remove it from
        // the table view and throw it in the recycle bin
        
        CGRect cellBounds = [cell bounds];
        
        if (CGRectIntersectsRect(viewBounds, cellBounds) == NO)
        {
            // The cell is no longer within the visible portion of the scroll view
            [cell removeFromSuperview];
            [recycledCells addObject:cell];
        }
    }
    
    
    // Determine how many rows are visible, given the row height
    NSInteger visibleRows = ( viewBounds.size.height / rowHeight ) + 1;
    
    // Remove any recycled objects from the visibleCell array
    [visibleCells removeObjectsInArray:recycledCells];
    
    // Create cells for the visible rows only...    
    CGFloat originY = viewBounds.origin.y; // need to fix this to account for scrolling in either direction
                                            // and the visibility of existing cells, etc.
    for ( NSInteger i = 0; i < visibleRows; i++) {
        // Get a cell ... it might be recycled or new. See dequeueReusableCellWithIdentifier below
        KRTableViewCell *cell = [tableDataSource tableView:self
                                     cellForRowAtIndexPath:[NSIndexPath indexPathForItem:i inSection:1]];
        
        // Add the cell into the "table"
        [cell setFrame:CGRectMake(0, originY, viewBounds.size.width, rowHeight)];
        originY += rowHeight;
        
        [self addSubview:cell];
        [visibleCells addObject:cell];
    }
}

// Method to find an existing cell object that can be reused

- (id)dequeueReusableCellWithIdentifier:(NSString *)identifier
{
    id cell = [recycledCells lastObject];
    if (cell != nil) {
        //NSLog(@"A recycled cell is available");
        [recycledCells removeObject:cell];
        [cell prepareForReuse];
    } else {
        NSLog(@"An existing cell is not available");
    }
    return cell;
}
/*
- (NSIndexPath *)indexPathForCell:(KRTableViewCell *)cell
{
    NSIndexPath *path = [NSIndexPath indexPathWithIndex:[visibleCells indexOfObject:cell]];
    return path;
}

- (KRTableViewCell *)cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    KRTableViewCell *cell = [visibleCells objectAtIndex:[indexPath row]];
    return cell;
}
*/

#pragma mark - UIScrollViewDelegate methods

- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
    // update the cells here?
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
}

- (void)scrollViewWillEndDragging:(UIScrollView *)scrollView withVelocity:(CGPoint)velocity targetContentOffset:(inout CGPoint *)targetContentOffset
{
}

- (void)scrollViewDidEndScrollingAnimation:(UIScrollView *)scrollView
{
}

@end
