//
//  KRViewController.m
//  CustomTableView
//
//  Created by Ken Runyon on 8/9/13.
//  Copyright (c) 2013 Ken Runyon. All rights reserved.
//

#import "KRViewController.h"
#import "KRTableView.h"
#import "KRTableViewCell.h"

@interface KRViewController ()

@property (nonatomic, readwrite) CGFloat rowHeight;

@end

@implementation KRViewController

@synthesize customTableView;
@synthesize rowHeight;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc
{
    // The custom table's delegate and dataSource members should take care of
    // themselves in the table view's dealloc method.
    
    // Release the backing data source
    dataSource = nil;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.
    
    // Create the table view control covering the entire view
    customTableView = [[KRTableView alloc] initWithFrame:[[self view] bounds]];
    [[self view] addSubview:customTableView];
    

    // Get the default system row height for a standard cell
    UITableViewCell *tmpCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
                                                      reuseIdentifier:@"GetDefaultRowHeight"];
    rowHeight = tmpCell.frame.size.height;
    tmpCell = nil;

    
    // Create the backing data source for the table
    dataSource = [[NSMutableArray alloc] init];
    
    // Create some data to display as rows
    for ( NSUInteger u = 0; u < 50; u++ ) {
        // Snagged these lines from the web to generate a random color object
        CGFloat hue = ( arc4random() % 256 / 256.0 );  //  0.0 to 1.0
        CGFloat saturation = ( arc4random() % 128 / 256.0 ) + 0.5;  //  0.5 to 1.0, away from white
        CGFloat brightness = ( arc4random() % 128 / 256.0 ) + 0.5;  //  0.5 to 1.0, away from black
        UIColor *randomColor = [UIColor colorWithHue:hue saturation:saturation brightness:brightness alpha:1];
        
        [dataSource addObject:randomColor];
    }
    
    customTableView.tableDelegate = self;
    customTableView.tableDataSource = self;
    
    [customTableView reloadData];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - KRTableViewDelegate Methods

- (CGFloat)tableView:(KRTableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return rowHeight;
}

#pragma mark - KRTableViewDataSource Methods

- (NSInteger)tableView:(KRTableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [dataSource count];
}

- (KRTableViewCell *)tableView:(KRTableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString * const cellIdentifier = @"CustomCell";
    
    KRTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if (cell == nil) {
        //NSLog(@"Creating a new table cell");
        cell = [[KRTableViewCell alloc] init];
    }
    
    UIColor *someRandomBackgroundColor = [dataSource objectAtIndex:[indexPath row]];
    
    [cell setBackgroundColor:someRandomBackgroundColor];

    return cell;
}


@end
